@@ -10,59 +10,65 @@
 from __future__ import absolute_import
 
 import os
+
+from mercurial.i18n import _
 from mercurial import (
     demandimport,
-    error
+    error,
+    pycompat,
 )
-from mercurial.i18n import _
 from . import common
 
 # these do not work with demandimport, blacklist
-demandimport.ignore.extend([
-        'bzrlib.transactions',
-        'bzrlib.urlutils',
-        'ElementPath',
-    ])
+demandimport.IGNORES.update(
+    [b'bzrlib.transactions', b'bzrlib.urlutils', b'ElementPath',]
+)
 
 try:
     # bazaar imports
     import bzrlib.bzrdir
     import bzrlib.errors
     import bzrlib.revision
-    import bzrlib.revisionspec.RevisionSpec
+    import bzrlib.revisionspec
+
     bzrdir = bzrlib.bzrdir
     errors = bzrlib.errors
     revision = bzrlib.revision
     revisionspec = bzrlib.revisionspec
+    revisionspec.RevisionSpec
 except ImportError:
     pass
 
-supportedkinds = ('file', 'symlink')
+supportedkinds = (b'file', b'symlink')
+
 
 class bzr_source(common.converter_source):
     """Reads Bazaar repositories by using the Bazaar Python libraries"""
 
-    def __init__(self, ui, path, revs=None):
-        super(bzr_source, self).__init__(ui, path, revs=revs)
+    def __init__(self, ui, repotype, path, revs=None):
+        super(bzr_source, self).__init__(ui, repotype, path, revs=revs)
 
-        if not os.path.exists(os.path.join(path, '.bzr')):
-            raise common.NoRepo(_('%s does not look like a Bazaar repository')
-                              % path)
+        if not os.path.exists(os.path.join(path, b'.bzr')):
+            raise common.NoRepo(
+                _(b'%s does not look like a Bazaar repository') % path
+            )
 
         try:
             # access bzrlib stuff
             bzrdir
         except NameError:
-            raise common.NoRepo(_('Bazaar modules could not be loaded'))
+            raise common.NoRepo(_(b'Bazaar modules could not be loaded'))
 
         path = os.path.abspath(path)
         self._checkrepotype(path)
         try:
             self.sourcerepo = bzrdir.BzrDir.open(path).open_repository()
         except errors.NoRepositoryPresent:
-            raise common.NoRepo(_('%s does not look like a Bazaar repository')
-                              % path)
+            raise common.NoRepo(
+                _(b'%s does not look like a Bazaar repository') % path
+            )
         self._parentids = {}
+        self._saverev = ui.configbool(b'convert', b'bzr.saverev')
 
     def _checkrepotype(self, path):
         # Lightweight checkouts detection is informational but probably
@@ -75,13 +81,20 @@ class bzr_source(common.converter_source):
             except (errors.NoWorkingTree, errors.NotLocalUrl):
                 tree = None
                 branch = dir.open_branch()
-            if (tree is not None and tree.bzrdir.root_transport.base !=
-                branch.bzrdir.root_transport.base):
-                self.ui.warn(_('warning: lightweight checkouts may cause '
-                               'conversion failures, try with a regular '
-                               'branch instead.\n'))
+            if (
+                tree is not None
+                and tree.bzrdir.root_transport.base
+                != branch.bzrdir.root_transport.base
+            ):
+                self.ui.warn(
+                    _(
+                        b'warning: lightweight checkouts may cause '
+                        b'conversion failures, try with a regular '
+                        b'branch instead.\n'
+                    )
+                )
         except Exception:
-            self.ui.note(_('bzr source type could not be determined\n'))
+            self.ui.note(_(b'bzr source type could not be determined\n'))
 
     def before(self):
         """Before the conversion begins, acquire a read lock
@@ -116,16 +129,17 @@ class bzr_source(common.converter_source):
                     pass
                 revid = info.rev_id
             if revid is None:
-                raise error.Abort(_('%s is not a valid revision')
-                                  % self.revs[0])
+                raise error.Abort(
+                    _(b'%s is not a valid revision') % self.revs[0]
+                )
             heads = [revid]
         # Empty repositories return 'null:', which cannot be retrieved
-        heads = [h for h in heads if h != 'null:']
+        heads = [h for h in heads if h != b'null:']
         return heads
 
     def getfile(self, name, rev):
         revtree = self.sourcerepo.revision_tree(rev)
-        fileid = revtree.path2id(name.decode(self.encoding or 'utf-8'))
+        fileid = revtree.path2id(name.decode(self.encoding or b'utf-8'))
         kind = None
         if fileid is not None:
             kind = revtree.kind(fileid)
@@ -133,11 +147,12 @@ class bzr_source(common.converter_source):
             # the file is not available anymore - was deleted
             return None, None
         mode = self._modecache[(name, rev)]
-        if kind == 'symlink':
+        if kind == b'symlink':
             target = revtree.get_symlink_target(fileid)
             if target is None:
-                raise error.Abort(_('%s.%s symlink has no target')
-                                 % (name, rev))
+                raise error.Abort(
+                    _(b'%s.%s symlink has no target') % (name, rev)
+                )
             return target, mode
         else:
             sio = revtree.get_file(fileid)
@@ -145,7 +160,7 @@ class bzr_source(common.converter_source):
 
     def getchanges(self, version, full):
         if full:
-            raise error.Abort(_("convert from cvs does not support --full"))
+            raise error.Abort(_(b"convert from cvs does not support --full"))
         self._modecache = {}
         self._revtree = self.sourcerepo.revision_tree(version)
         # get the parentids from the cache
@@ -165,15 +180,18 @@ class bzr_source(common.converter_source):
             parents = self._filterghosts(rev.parent_ids)
             self._parentids[version] = parents
 
-        branch = self.recode(rev.properties.get('branch-nick', u'default'))
-        if branch == 'trunk':
-            branch = 'default'
-        return common.commit(parents=parents,
-                date='%d %d' % (rev.timestamp, -rev.timezone),
-                author=self.recode(rev.committer),
-                desc=self.recode(rev.message),
-                branch=branch,
-                rev=version)
+        branch = self.recode(rev.properties.get(b'branch-nick', u'default'))
+        if branch == b'trunk':
+            branch = b'default'
+        return common.commit(
+            parents=parents,
+            date=b'%d %d' % (rev.timestamp, -rev.timezone),
+            author=self.recode(rev.committer),
+            desc=self.recode(rev.message),
+            branch=branch,
+            rev=version,
+            saverev=self._saverev,
+        )
 
     def gettags(self):
         bytetags = {}
@@ -181,7 +199,7 @@ class bzr_source(common.converter_source):
             if not branch.supports_tags():
                 return {}
             tagdict = branch.tags.get_tag_dict()
-            for name, rev in tagdict.iteritems():
+            for name, rev in pycompat.iteritems(tagdict):
                 bytetags[self.recode(name)] = rev
         return bytetags
 
@@ -203,13 +221,30 @@ class bzr_source(common.converter_source):
         changes = []
         renames = {}
         seen = set()
+
+        # Fall back to the deprecated attribute for legacy installations.
+        try:
+            inventory = origin.root_inventory
+        except AttributeError:
+            inventory = origin.inventory
+
         # Process the entries by reverse lexicographic name order to
         # handle nested renames correctly, most specific first.
-        curchanges = sorted(current.iter_changes(origin),
-                            key=lambda c: c[1][0] or c[1][1],
-                            reverse=True)
-        for (fileid, paths, changed_content, versioned, parent, name,
-            kind, executable) in curchanges:
+        curchanges = sorted(
+            current.iter_changes(origin),
+            key=lambda c: c[1][0] or c[1][1],
+            reverse=True,
+        )
+        for (
+            fileid,
+            paths,
+            changed_content,
+            versioned,
+            parent,
+            name,
+            kind,
+            executable,
+        ) in curchanges:
 
             if paths[0] == u'' or paths[1] == u'':
                 # ignore changes to tree root
@@ -217,23 +252,23 @@ class bzr_source(common.converter_source):
 
             # bazaar tracks directories, mercurial does not, so
             # we have to rename the directory contents
-            if kind[1] == 'directory':
-                if kind[0] not in (None, 'directory'):
+            if kind[1] == b'directory':
+                if kind[0] not in (None, b'directory'):
                     # Replacing 'something' with a directory, record it
                     # so it can be removed.
                     changes.append((self.recode(paths[0]), revid))
 
-                if kind[0] == 'directory' and None not in paths:
+                if kind[0] == b'directory' and None not in paths:
                     renaming = paths[0] != paths[1]
                     # neither an add nor an delete - a move
                     # rename all directory contents manually
-                    subdir = origin.inventory.path2id(paths[0])
+                    subdir = inventory.path2id(paths[0])
                     # get all child-entries of the directory
-                    for name, entry in origin.inventory.iter_entries(subdir):
+                    for name, entry in inventory.iter_entries(subdir):
                         # hg does not track directory renames
-                        if entry.kind == 'directory':
+                        if entry.kind == b'directory':
                             continue
-                        frompath = self.recode(paths[0] + '/' + name)
+                        frompath = self.recode(paths[0] + b'/' + name)
                         if frompath in seen:
                             # Already handled by a more specific change entry
                             # This is important when you have:
@@ -244,14 +279,16 @@ class bzr_source(common.converter_source):
                         seen.add(frompath)
                         if not renaming:
                             continue
-                        topath = self.recode(paths[1] + '/' + name)
+                        topath = self.recode(paths[1] + b'/' + name)
                         # register the files as changed
                         changes.append((frompath, revid))
                         changes.append((topath, revid))
                         # add to mode cache
-                        mode = ((entry.executable and 'x')
-                                or (entry.kind == 'symlink' and 's')
-                                or '')
+                        mode = (
+                            (entry.executable and b'x')
+                            or (entry.kind == b'symlink' and b's')
+                            or b''
+                        )
                         self._modecache[(topath, revid)] = mode
                         # register the change as move
                         renames[topath] = frompath
@@ -279,8 +316,7 @@ class bzr_source(common.converter_source):
 
             # populate the mode cache
             kind, executable = [e[1] for e in (kind, executable)]
-            mode = ((executable and 'x') or (kind == 'symlink' and 'l')
-                    or '')
+            mode = (executable and b'x') or (kind == b'symlink' and b'l') or b''
             self._modecache[(topath, revid)] = mode
             changes.append((topath, revid))
 
