@@ -11,12 +11,15 @@ import errno
 import os
 
 from .i18n import _
+from .pycompat import getattr
 from . import (
+    encoding,
     error,
     pycompat,
     util,
 )
 
+
 class config(object):
     def __init__(self, data=None, includepaths=None):
         self._data = {}
@@ -28,17 +31,23 @@ class config(object):
             self._source = data._source.copy()
         else:
             self._source = util.cowdict()
+
     def copy(self):
         return config(self)
+
     def __contains__(self, section):
         return section in self._data
+
     def hasitem(self, section, item):
         return item in self._data.get(section, {})
+
     def __getitem__(self, section):
         return self._data.get(section, {})
+
     def __iter__(self):
         for d in self.sections():
             yield d
+
     def update(self, src):
         self._source = self._source.preparewrite()
         for s, n in src._unset:
@@ -55,6 +64,7 @@ class config(object):
                 self._data[s] = util.cowsortdict()
             self._data[s].update(src._data[s])
         self._source.update(src._source)
+
     def get(self, section, item, default=None):
         return self._data.get(section, {}).get(item, default)
 
@@ -71,15 +81,25 @@ class config(object):
             return (section, item)
 
     def source(self, section, item):
-        return self._source.get((section, item), "")
+        return self._source.get((section, item), b"")
+
     def sections(self):
         return sorted(self._data.keys())
+
     def items(self, section):
-        return list(self._data.get(section, {}).iteritems())
-    def set(self, section, item, value, source=""):
+        return list(pycompat.iteritems(self._data.get(section, {})))
+
+    def set(self, section, item, value, source=b""):
         if pycompat.ispy3:
-            assert not isinstance(value, str), (
-                'config values may not be unicode strings on Python 3')
+            assert not isinstance(
+                section, str
+            ), b'config section may not be unicode strings on Python 3'
+            assert not isinstance(
+                item, str
+            ), b'config item may not be unicode strings on Python 3'
+            assert not isinstance(
+                value, str
+            ), b'config values may not be unicode strings on Python 3'
         if section not in self:
             self._data[section] = util.cowsortdict()
         else:
@@ -113,7 +133,7 @@ class config(object):
         commentre = util.re.compile(br'(;|#)')
         unsetre = util.re.compile(br'%unset\s+(\S+)')
         includere = util.re.compile(br'%include\s+(\S|\S.*\S)\s*$')
-        section = ""
+        section = b""
         item = None
         line = 0
         cont = False
@@ -123,7 +143,7 @@ class config(object):
 
         for l in data.splitlines(True):
             line += 1
-            if line == 1 and l.startswith('\xef\xbb\xbf'):
+            if line == 1 and l.startswith(b'\xef\xbb\xbf'):
                 # Someone set us up the BOM
                 l = l[3:]
             if cont:
@@ -133,8 +153,8 @@ class config(object):
                 if m:
                     if sections and section not in sections:
                         continue
-                    v = self.get(section, item) + "\n" + m.group(1)
-                    self.set(section, item, v, "%s:%d" % (src, line))
+                    v = self.get(section, item) + b"\n" + m.group(1)
+                    self.set(section, item, v, b"%s:%d" % (src, line))
                     continue
                 item = None
                 cont = False
@@ -152,9 +172,11 @@ class config(object):
                         break
                     except IOError as inst:
                         if inst.errno != errno.ENOENT:
-                            raise error.ParseError(_("cannot include %s (%s)")
-                                                   % (inc, inst.strerror),
-                                                   "%s:%s" % (src, line))
+                            raise error.ParseError(
+                                _(b"cannot include %s (%s)")
+                                % (inc, encoding.strtolocal(inst.strerror)),
+                                b"%s:%d" % (src, line),
+                            )
                 continue
             if emptyre.match(l):
                 continue
@@ -172,7 +194,7 @@ class config(object):
                 cont = True
                 if sections and section not in sections:
                     continue
-                self.set(section, item, m.group(2), "%s:%d" % (src, line))
+                self.set(section, item, m.group(2), b"%s:%d" % (src, line))
                 continue
             m = unsetre.match(l)
             if m:
@@ -185,16 +207,19 @@ class config(object):
                 self._unset.append((section, name))
                 continue
 
-            raise error.ParseError(l.rstrip(), ("%s:%s" % (src, line)))
+            raise error.ParseError(l.rstrip(), (b"%s:%d" % (src, line)))
 
     def read(self, path, fp=None, sections=None, remap=None):
         if not fp:
-            fp = util.posixfile(path, 'rb')
-        assert getattr(fp, 'mode', r'rb') == r'rb', (
-            'config files must be opened in binary mode, got fp=%r mode=%r' % (
-                fp, fp.mode))
-        self.parse(path, fp.read(),
-                   sections=sections, remap=remap, include=self.read)
+            fp = util.posixfile(path, b'rb')
+        assert getattr(fp, 'mode', 'rb') == 'rb', (
+            b'config files must be opened in binary mode, got fp=%r mode=%r'
+            % (fp, fp.mode,)
+        )
+        self.parse(
+            path, fp.read(), sections=sections, remap=remap, include=self.read
+        )
+
 
 def parselist(value):
     """parse a configuration value as a list of comma/space separated strings
@@ -205,76 +230,82 @@ def parselist(value):
 
     def _parse_plain(parts, s, offset):
         whitespace = False
-        while offset < len(s) and (s[offset:offset + 1].isspace()
-                                   or s[offset:offset + 1] == ','):
+        while offset < len(s) and (
+            s[offset : offset + 1].isspace() or s[offset : offset + 1] == b','
+        ):
             whitespace = True
             offset += 1
         if offset >= len(s):
             return None, parts, offset
         if whitespace:
-            parts.append('')
-        if s[offset:offset + 1] == '"' and not parts[-1]:
+            parts.append(b'')
+        if s[offset : offset + 1] == b'"' and not parts[-1]:
             return _parse_quote, parts, offset + 1
-        elif s[offset:offset + 1] == '"' and parts[-1][-1] == '\\':
-            parts[-1] = parts[-1][:-1] + s[offset:offset + 1]
+        elif s[offset : offset + 1] == b'"' and parts[-1][-1:] == b'\\':
+            parts[-1] = parts[-1][:-1] + s[offset : offset + 1]
             return _parse_plain, parts, offset + 1
-        parts[-1] += s[offset:offset + 1]
+        parts[-1] += s[offset : offset + 1]
         return _parse_plain, parts, offset + 1
 
     def _parse_quote(parts, s, offset):
-        if offset < len(s) and s[offset:offset + 1] == '"': # ""
-            parts.append('')
+        if offset < len(s) and s[offset : offset + 1] == b'"':  # ""
+            parts.append(b'')
             offset += 1
-            while offset < len(s) and (s[offset:offset + 1].isspace() or
-                    s[offset:offset + 1] == ','):
+            while offset < len(s) and (
+                s[offset : offset + 1].isspace()
+                or s[offset : offset + 1] == b','
+            ):
                 offset += 1
             return _parse_plain, parts, offset
 
-        while offset < len(s) and s[offset:offset + 1] != '"':
-            if (s[offset:offset + 1] == '\\' and offset + 1 < len(s)
-                    and s[offset + 1:offset + 2] == '"'):
+        while offset < len(s) and s[offset : offset + 1] != b'"':
+            if (
+                s[offset : offset + 1] == b'\\'
+                and offset + 1 < len(s)
+                and s[offset + 1 : offset + 2] == b'"'
+            ):
                 offset += 1
-                parts[-1] += '"'
+                parts[-1] += b'"'
             else:
-                parts[-1] += s[offset:offset + 1]
+                parts[-1] += s[offset : offset + 1]
             offset += 1
 
         if offset >= len(s):
             real_parts = _configlist(parts[-1])
             if not real_parts:
-                parts[-1] = '"'
+                parts[-1] = b'"'
             else:
-                real_parts[0] = '"' + real_parts[0]
+                real_parts[0] = b'"' + real_parts[0]
                 parts = parts[:-1]
                 parts.extend(real_parts)
             return None, parts, offset
 
         offset += 1
-        while offset < len(s) and s[offset:offset + 1] in [' ', ',']:
+        while offset < len(s) and s[offset : offset + 1] in [b' ', b',']:
             offset += 1
 
         if offset < len(s):
-            if offset + 1 == len(s) and s[offset:offset + 1] == '"':
-                parts[-1] += '"'
+            if offset + 1 == len(s) and s[offset : offset + 1] == b'"':
+                parts[-1] += b'"'
                 offset += 1
             else:
-                parts.append('')
+                parts.append(b'')
         else:
             return None, parts, offset
 
         return _parse_plain, parts, offset
 
     def _configlist(s):
-        s = s.rstrip(' ,')
+        s = s.rstrip(b' ,')
         if not s:
             return []
-        parser, parts, offset = _parse_plain, [''], 0
+        parser, parts, offset = _parse_plain, [b''], 0
         while parser:
             parser, parts, offset = parser(parts, s, offset)
         return parts
 
     if value is not None and isinstance(value, bytes):
-        result = _configlist(value.lstrip(' ,\n'))
+        result = _configlist(value.lstrip(b' ,\n'))
     else:
         result = value
     return result or []
